use std::{fs, time::Duration};

use anyhow::{bail, Context};
use serde::Deserialize;
use serde_with::{serde_as, DurationSeconds};

#[derive(Deserialize, Clone)]
pub struct FelinetConfig {
    pub address: String,
}

#[serde_as]
#[derive(Deserialize, Clone)]
pub struct BeaconConfig {
    #[serde_as(as = "DurationSeconds<u64>")]
    pub period_seconds: Duration,
    #[serde(default)]
    pub max_hops: u8,
    pub latitude: Option<f64>,
    pub longitude: Option<f64>,
    pub altitude: Option<f64>,
    pub comment: Option<String>,
}

#[derive(Deserialize, Clone)]
pub struct Config {
    pub callsign: String,
    pub ssid: u8,
    #[serde(default)]
    pub icon: u16,
    pub felinet: FelinetConfig,
    pub beacon: BeaconConfig,
}

impl Config {
    pub fn load() -> anyhow::Result<Self> {
        let file_contents = read_conf_file()?;
        let config: Self = toml::from_str(&file_contents).context("parsing config file")?;

        if config.callsign == "CHANGEME" {
            bail!("Callsign is not configured in config file");
        }

        Ok(config)
    }
}

fn read_conf_file() -> anyhow::Result<String> {
    let filenames = ["config.toml", "/etc/cats_igate/config.toml"];
    for filename in filenames.iter() {
        if let Ok(x) = fs::read_to_string(filename) {
            return Ok(x);
        }
    }

    bail!("Could not open config file. Should either be `config.toml` in the current directory, or in /etc/cats_igate/")
}
